/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Log management
*  SCOPE :              Log management
*  FILE :               log.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __rename_H
#include "rename.h"
#endif

#ifndef __fnctl_H
#define __fnctl_H
#include <fcntl.h>
#endif

#include <string.h>

#include <sys/stat.h>

#ifndef __log_HXX
#include "log.hxx"
#endif

#ifndef __config_H
#include "config.h"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

/*****************************************************************************/

const int Log::MapSize = yoo_alignSup(MAX_LOG*sizeof(PID), PAGE_SIZE) ;

/*= EXTERNAL FUNCTION =======================================================*/
Log::Log(char* f) : _file(strdup(f)) 
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{  
  if (access(_file, 0) == 0) {
    SYSCALLI(this->_fd = open(_file, O_RDWR)) ;
    struct stat buf ;
    SYSCALLI(fstat(this->_fd, &buf)) ;
    this->_size = ((int) buf.st_size - MapSize - PAGE_SIZE) / PAGE_SIZE ;
    _inuse = (int*)yoo_mmap(0,PAGE_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,_fd,0);
  } else {
    SYSCALLI(this->_fd = open(_file, O_CREAT|O_RDWR, 0666)) ;
    SYSCALLI(ftruncate(this->_fd, MapSize + PAGE_SIZE)) ;
    this->_size = 0 ;
    _inuse = (int*)yoo_mmap(0,PAGE_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,_fd,0);
    *_inuse = TRUE ;
  }

  _addresses = (PID *)yoo_mmap(0, MapSize, PROT_READ|PROT_WRITE, MAP_SHARED,
			       _fd, PAGE_SIZE);
}


/*= EXTERNAL FUNCTION =======================================================*/
PID Log::read(Page* p, int pagenum)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  SYSCALLI(lseek(this->_fd, (pagenum*PAGE_SIZE)+MapSize+PAGE_SIZE, SEEK_SET));
  SYSCALLI(::read(this->_fd, p, PAGE_SIZE)) ;
  return _addresses[pagenum] ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int Log:: write(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  SYSCALLI(lseek(this->_fd,(_size*PAGE_SIZE)+MapSize+PAGE_SIZE, SEEK_SET));
  _addresses[_size] = pid ;
  SYSCALLI(::write(this->_fd, p, PAGE_SIZE)) ;
  return _size++ ;
}



